--
-- This file implements the EWT API using the LuaBox API
-- The idea is that we can now code using just the EWT API
-- and both LB and EWT will "just work"
--

--- Move to a position.
-- X           (number)  - The X coordinate
-- Y           (number)  - The Y coordinate
-- Z           (number)  - The Z coordinate
-- InstantTurn (boolean) - Whether to instantly turn toward the destination
function MoveTo (X, Y, Z, InstantTurn)
  __LB__.MoveTo(X, Y, Z)
end

--- Get the names of the files in a directory.
-- Path    (string) - The path to the files
-- returns (table)  - The file names
-- Example: "C:\*" would retrieve the names of all of the files in C:\.
-- Example: "C:\*.dll" would retrieve the names of all of the .dll files in C:\.
function GetDirectoryFiles (Path)
  return __LB__.GetFiles(Path)
end

--- Get the contents of a text file.
-- Path    (string) - The file path
-- Binary  (bool)   - Whether the file contents should be returned as a table of bytes
-- returns (string or table) - The file contents
-- Note: The table contains unsigned numbers, so {0x41, 0x42, 0x43, 0x85, 0xFE, -5, -10} will return {65, 66, 67, 133, 254, 251, 246}
-- function ReadFile (Path [, AsBinary])
function ReadFile (Path, AsBinary)
  return __LB__.ReadFile(Path)
end

--- Set the contents of or append a text file.
-- Path   (string)  - The file path
-- Contents (string or table)  - The file contents as a string or table of bytes
-- Append (boolean) - Whether to append rather than overwrite
-- CreatePath (boolean) - Whether to create directories for the given file path.
-- Note: If you provide just a filename in Path, then it will create the file where Wow.exe is located.
function WriteFile (Path, Contents, Append, CreatePath)
  return __LB__.WriteFile(Path, Contents, Append)
end

--- Return whether the given coordinates are in front of WoW's camera
-- X        (number)  - The X coordinate.
-- Y        (number)  - The Y coordinate.
-- Z        (number)  - The Z coordinate.
-- returns  (boolean) - Whether they are in front or not.
-- Note: This API isn't really needed if you are already calling WorldToScreen.
function IsInFront (X, Y, Z)
  return __LB__.WorldToScreen(X, Y, Z) ~= nil
end

--- Gets the active player.
-- returns (object or nil) - The active player object or nil if it does not exist.
function GetActivePlayer()
  return __LB__.ObjectPointer('player')
end

--- Get the distance between two objects.
-- Object1 (object) - The first object
-- Object2 (object) - The second object
-- returns (number) - The distance
function GetDistanceBetweenObjects (Object1, Object2)
  return __LB__.GetDistance3D(Object1, Object2)
end

--- Get the number of objects in the object manager and optionally 2 tables with all objects that were updated in the frame.
-- GetUpdates (boolean)  - True to get the tables with the objects that were updated in the frame.
-- Id (string)           - A simple string identifier to avoid conflicts between multiple addons calling GetObjectCount(true) in the same frame.
-- returns (integer, boolean, table, table) - The total number of objects, if the object manager was updated, the added objects and the removed objects.
-- Note: This should be the first API that you should call before using the other object manager functions.
-- Ideally, call it once every frame with an OnUpdate frame event.
-- You can use table.getn or # to get the number of objects in the returned tables.
-- Be careful with caching: the object manager can be updated after your GetObjectCount call within the same frame.
local Objects = nil
function GetObjectCount()
  local tmp = __LB__.GetObjects()
  Objects = { }
  for _, obj in ipairs(tmp) do
    if obj ~= nil then
      table.insert(Objects, obj)
    end
  end
  return #Objects
end
ObjectCount = GetObjectCount

--- Get an object's creator.
-- Object  (object) - The unit or game object
-- returns (unit) - The creator or nil if there is none
function ObjectCreator(Unit)
  return __LB__.ObjectCreator(Unit)
end
UnitCreator = ObjectCreator

--- Get an object in the object manager from its index.
-- Index   (integer) - The one-based index of the object. Starts at 1.
-- returns (object)  - The object
-- Note: If an object is at index 30, this does NOT mean that in the next frame it will be at index 30 again.
-- This API isn't really needed if you are properly using GetObjectCount(true).
function GetObjectWithIndex(Index)
  return Objects[Index]
end
ObjectWithIndex = GetObjectWithIndex

--- Get the distance between two objects.
-- Object1 (object) - The first object
-- Object2 (object) - The second object
-- returns (number) - The distance
function GetDistanceBetweenObjects(Object1, Object2)
  return __LB__.GetDistance3D(Object1, Object2)
end

--- Get an object's name.
-- Object  (object) - The object
-- returns (string) - The name
function ObjectName(Object)
  return __LB__.ObjectName(Object)
end

--- Get whether an object exists in the object manager with O(n) performance.
-- Object  (object)  - The object
-- returns (string or nil) - The pointer as a hexadecimal string prefixed by 0x or nil if the object does not exist.
-- Note that if the object does not exist in the object manager it is invalid and should not be used.
-- This API has O(n) performance, which means that the more objects in the object manager, the slower it is.
-- This API is only useful to check if '0x' strings are *really* valid. It should never be used
-- if you are properly using GetObjectCount since you will always work with objects that exist.
-- This API is similar to ObjectPointer, but it checks if the object exists in the object manager.
-- If you pass a unitID, like ObjectExists('target'), then its performance is the same as ObjectPointer('target').
function ObjectExists (Object)
  return __LB__.ObjectExists(Object)
end

--- Get an object's type ID.
-- Object  (object)  - The object
-- returns (integer) - The type ID or nil if the object is invalid
function ObjectID (Object)
  return __LB__.ObjectId(Object)
end
